// memoryShapes are forcedShapes for PR and allowedShapes for CR
// note for CR all but one of memoryShapes can be used.
import java.util.*;

class FeasibleSoln{
    private LinkedList shapes;
    private LinkedList shapeValues;
    
    private LinkedList memoryShapes;     // stores ShapeMatrices
    private LinkedList memoryShapeValues; // stores Integers
    private boolean [] usedAllowedShapes; // indicies are same as allowedShapes
                                          // only need for comp region
    public FeasibleSoln(){
        shapes= new LinkedList();
        shapeValues= new LinkedList();
        memoryShapes= new LinkedList();
        memoryShapeValues= new LinkedList();
        
        usedAllowedShapes= new boolean[1000];
        for (int i = 0; i < memoryShapes.size(); i++){
            usedAllowedShapes[i]=false;
        }
    }
    
    public int numShapes(){
        return shapes.size();
    }
    
    public ShapeMatrix getShape(int i){
        return (ShapeMatrix)(shapes.get(i));
    }

    // return's all shapes, including duplicates
    public LinkedList getShapesPromRegion(){
        LinkedList tmp= new LinkedList();
        for (int i = 0; i < memoryShapes.size(); i++)
            tmp.add(memoryShapes.get(i));		  
        for (int i = 0; i < shapes.size(); i++)
            tmp.add(shapes.get(i));
        return tmp;
    }			
    
    public LinkedList getShapesPromRegionNoDups(){
        LinkedList tmp= new LinkedList();
        //System.err.println("size of memory shapes is "+ memoryShapes.size());
        for (int i = 0; i < memoryShapes.size(); i++)
            tmp.add(memoryShapes.get(i));
        for (int i = 0; i < shapes.size(); i++)
            tmp.add(shapes.get(i));

        for (int i = 0; i < tmp.size(); i++){
            ShapeMatrix m= (ShapeMatrix) (tmp.get(i));
            for (int j = i+1; j < tmp.size(); j++){
                ShapeMatrix n= (ShapeMatrix) (tmp.get(j));
                if (m.equal(n)){
                    System.err.println("Removing a shape");
                    //System.err.println(m);                    
                    //System.err.println(n);                    
                    tmp.remove(j);
                }
            }
        }
        //System.err.println("The number of shapes added to tmp is " + tmp.size());
        return tmp;
    }		        
        
     public LinkedList getShapesCompRegion(){
        return shapes;
    }
     
    public LinkedList getShapesCompRegionNoDups(){
        LinkedList tmp= new LinkedList();
        for (int i = 0; i < shapes.size(); i++)
            tmp.add(shapes.get(i));
        for (int i = 0; i < tmp.size(); i++){
            ShapeMatrix m= (ShapeMatrix) (tmp.get(i));
            for (int j= i+1; j < tmp.size(); j++){
                ShapeMatrix n= (ShapeMatrix) (tmp.get(j));
                if (m.equal(n)){
                    System.err.println("Removing a shape");
                    //System.err.println(m);                    
                    //System.err.println(n);                    
                    tmp.remove(j);                    
                }
            }
        }
        return tmp;
    }
            
    public int getShapeValue(int i){
        return ((Integer)(shapeValues.get(i))).intValue();
    }
    
    public void addShape(ShapeMatrix s, int shapeValue){
        shapes.add(s);
        shapeValues.add(new Integer(shapeValue));
    }
    
    public void removeFirstShape(){
        shapes.removeFirst();
        shapeValues.removeFirst();
    }
    
    public void removeShape(int i){
        shapes.remove(i);
        shapeValues.remove(i);
    }
    
    public int getBOTmemory(){
        int beamOT=0;
        for (int i = 0; memoryShapeValues!=null && i < memoryShapeValues.size(); i++)
            beamOT+= ((Integer)(memoryShapeValues.get(i))).intValue();
        return beamOT;
    }
    public int getBOTshapes(){
        int beamOT=0;
        for (int i = 0; shapeValues!=null && i < shapeValues.size(); i++)
            beamOT+= ((Integer)(shapeValues.get(i))).intValue();
        return beamOT;
    }
    
    public void addMemoryShapes(LinkedList ll){
        memoryShapes= ll;
    }
    
    public void addMemoryShapeValues(LinkedList ll){
        memoryShapeValues= ll;
    }
   
    public void addMemoryShape(ShapeMatrix s, int shapeValue){
        memoryShapes.add(s);
        memoryShapeValues.add(new Integer(shapeValue));
    }
    
    public LinkedList getMemoryShapes(){
        return memoryShapes;
    }
    
    public ShapeMatrix getMemoryShape(int i){
        return (ShapeMatrix)(memoryShapes.get(i));
    }
    
    public LinkedList getMemoryShapeValues(){
        return memoryShapeValues;
    }
    
    public int getMemoryShapeValue(int i){
        return ((Integer)(memoryShapeValues.get(i))).intValue();
    }
    
    public int removeUnusedMemoryShapes(){
        int count=0;
            int arrayIndex=0;
        for (int i = 0; i < memoryShapes.size(); i++, arrayIndex++){
            System.out.println("i= " + i + " arrayIndex= " + arrayIndex);
            if (!usedAllowedShapes[arrayIndex]){
                System.out.println("**** Removing a memory Shape ****");
                System.out.println((ShapeMatrix)(memoryShapes.get(i)));
                count++;
                if (i == 0){
                    memoryShapes.removeFirst();
                    memoryShapeValues.removeFirst();
                }
                else{
                    memoryShapes.remove(i);
                    memoryShapeValues.remove(i);
                }
		i--;
            }
        }
        return count;
    }

	 public void reinitUsedAllowedShapes(){
		  for (int i = 0; i < memoryShapes.size()+1; i++)
				usedAllowedShapes[i]= false;
	 }

    // keep track of which allowed shapes, which forced shapes have been used
    // array of booleans
    public void updateShapeUsed(ShapeMatrix s){
        // check if an allowedShape
        for (int i = 0; i < memoryShapes.size(); i++)
            if (s.equal( (ShapeMatrix)(memoryShapes.get(i))) && !usedAllowedShapes[i]){
		System.out.println("This is a memory shape- recording it's use");
                usedAllowedShapes[i]= true;
                return;
            }
    }

    public boolean canUseShape(ShapeMatrix s){
        int memoryShapesUsed= 0;
            for (int i = 0; i < memoryShapes.size(); i++)
                if (usedAllowedShapes[i]){
                    memoryShapesUsed++;
                }
		if (memoryShapes.size() >  memoryShapesUsed)
                    return true;
		  System.out.println("Can't use the shape");
		  return false;
	 }

    public ShapeMatrix getDisallowedShape(){
        if (memoryShapes.size()==1)
            return (ShapeMatrix) memoryShapes.get(0);
        System.out.println("There are more than 1 memory shapes");
        int numMemoryShapesUsed=0;
        Object disallowUnusedMemoryShape= null;
        for (int i = 0; i < memoryShapes.size(); i++)
            if (usedAllowedShapes[i])
                numMemoryShapesUsed++;
            else
                disallowUnusedMemoryShape = memoryShapes.get(i);
        if (memoryShapes.size() ==  numMemoryShapesUsed+1)
            return (ShapeMatrix) disallowUnusedMemoryShape;
        else 
            return null;
    }
         
    public void printMemoryShapes(){
        for (int i = 0; i < memoryShapes.size(); i++){
            System.out.println("Printing Memory Shape " + i);
            System.out.println( (ShapeMatrix)(memoryShapes.get(i)));
        }
    }

    public void printShapes(){
        for (int i = 0; i < shapes.size(); i++){
            System.out.println("Printing Shape " + (memoryShapes.size()+i));
            System.out.println( (ShapeMatrix)(shapes.get(i)));
            }	 
    }
}
