import java.util.*;

// ----------------------------------------------------------------------------

// Represent a heap as an array. Start indices with 0, not 1.

class Heap {

    private Object[] heap;
    private int size     = 0;
    private int maxSize  = 16;
    Comparator comparator;

    // ------------------------------------------------------------------------

    public Heap(Comparator comparator1) {

	heap       = new Object[maxSize];
	comparator = comparator1;
    }

    // ------------------------------------------------------------------------

    public Heap(Comparator comparator1, int maxSize1) {

	maxSize    = maxSize1;
	heap       = new Object[maxSize];
	comparator = comparator1;
    }

    // ------------------------------------------------------------------------

    public boolean isEmpty() {

	return (size == 0);
    }

    // ------------------------------------------------------------------------

    public int size() {

	return size;
    }

    // ------------------------------------------------------------------------

    public boolean insert(Object object) {

	if (size == maxSize)
	    return false;

	heap[size++] = object;

	int parent = size - 1;

	while (true)

	    try {
		int child = parent;

		if (child != 0)
		    parent = (child - 1) / 2;

		if (child == 0)
		    break;

		if (comparator.compare(heap[parent], heap[child]) >= 0)
		    break;

		Object temp  = heap[child];
		heap[child]  = heap[parent];
		heap[parent] = temp;
	    }
	    catch (ClassCastException e) {
		fatalError(e);
	    }
	return true;
    }

    // ------------------------------------------------------------------------

    public Object remove() {

	if (size == 0)
	    return null;

	if (size == 1) {
	    size--;
	    return heap[0];
	}

	int maxChild = 0;
	Object root  = heap[0];
	heap[0]      = heap[--size];

	while (true)

	    try {
		int parent = maxChild;
		int left   = 2 * parent + 1;
		int right  = left + 1;

		if (right <= size - 1)
		    maxChild
			= (comparator.compare(heap[left], heap[right]) > 0) ?
			left : right;

		if (left == size - 1)
		    maxChild = left;

		if (left > size - 1)
		    break;

		if (comparator.compare(heap[parent], heap[maxChild]) >= 0)
		    break;

		Object temp    = heap[maxChild];
		heap[maxChild] = heap[parent];
		heap[parent]   = temp;
	    }
	    catch (ClassCastException e) {
		fatalError(e);
	    }
	return root;
    }

    // ------------------------------------------------------------------------

    public String toString() {

	String s = "";

	for (int n = 0; n < size; n++)

	    s += n + "\t" + heap[n] + "\n";

	return s;
    }

    // ------------------------------------------------------------------------

    private void fatalError(Object object) {

	System.out.println(object);
	System.exit(1);
    }
}
