import java.io.*;
import java.util.*;

// ----------------------------------------------------------------------------

class Matrix {

    Matrix() {}

    public static int[][] getIntensityMatrix(String inputFile) {

	int numUntrimmedRows = 0;
	int numUntrimmedCols = 0;
	int[][] untrimmedIntensityMatrix = null;

	try {
	    BufferedReader br = new BufferedReader(new FileReader(inputFile));

	    int numTokensOnLine1 = -1;
	    int lineNum;
	    String line;

	    // Make sure that each line has the same number of tokens.

	    for (lineNum = 0; (line = br.readLine()) != null; lineNum++) {

		// If a line is all whitespace, skip it.

		if (line.trim().equals("")) {
		    lineNum--;
		    continue;
		}

		StringTokenizer st
		    = new StringTokenizer(line, " \t\n\r\f,");

		int numTokens = st.countTokens();

		if (numTokensOnLine1 < 0)
		    numTokensOnLine1 = numTokens;

		else if (numTokensOnLine1 != numTokens)

		    fatalError("Line 1 has " + numTokensOnLine1
			       + " token"
			       + ((numTokensOnLine1 == 1) ? "" : "s")
			       + ", but line " + lineNum + " has "
			       + numTokens + " token"
			       + ((numTokens == 1) ? "" : "s") + ".");
	    }

	    // Try to read all tokens into a 2-D int array.

	    numUntrimmedRows             = lineNum;
	    numUntrimmedCols             = numTokensOnLine1;
	    untrimmedIntensityMatrix
		= new int[numUntrimmedRows + 1][numUntrimmedCols + 1];

	    br = new BufferedReader(new FileReader(inputFile));

	    for (int row = 1; row <= numUntrimmedRows; row++) {

		line = br.readLine();

		// If a line is all whitespace, skip it.

		if (line.trim().equals("")) {
		    row--;
		    continue;
		}

		StringTokenizer st
		    = new StringTokenizer(line, " \t\n\r\f,");

		for (int col = 1; col <= numUntrimmedCols; col++)

		    untrimmedIntensityMatrix[row][col]
			= Integer.parseInt(st.nextToken());
	    }
	}
	catch (FileNotFoundException e) {
	    fatalError(e);
	}
	catch (IOException e) {
	    fatalError(e);
	}
	catch (NumberFormatException e) {
	    fatalError(e);
	}

	// Find the first nonzero row in the intensity matrix.

	int firstNonzeroRow = -1;

	for (int row = 1; row <= numUntrimmedRows && firstNonzeroRow < 0;
	     row++)

	    for (int col = 1; col <= numUntrimmedCols && firstNonzeroRow < 0;
		 col++)

		if (untrimmedIntensityMatrix[row][col] != 0)
		    firstNonzeroRow = row;

	// Find the last nonzero row in the intensity matrix.

	int lastNonzeroRow = -1;

	for (int row = numUntrimmedRows; row >= 1 && lastNonzeroRow < 0; row--)

	    for (int col = 1; col <= numUntrimmedCols && lastNonzeroRow < 0;
		 col++)

		if (untrimmedIntensityMatrix[row][col] != 0)
		    lastNonzeroRow = row;

	// Find the first nonzero col in the intensity matrix.

	int firstNonzeroCol = -1;

	for (int col = 1; col <= numUntrimmedCols && firstNonzeroCol < 0;
	     col++)

	    for (int row = firstNonzeroRow;
		 row <= lastNonzeroRow && firstNonzeroCol < 0; row++)

		if (untrimmedIntensityMatrix[row][col] != 0)
		    firstNonzeroCol = col;

	// Find the last nonzero col in the intensity matrix.

	int lastNonzeroCol = -1;

	for (int col = numUntrimmedCols; col >= 1 && lastNonzeroCol < 0; col--)

	    for (int row = firstNonzeroRow;
		 row <= lastNonzeroRow && lastNonzeroCol < 0; row++)

		if (untrimmedIntensityMatrix[row][col] != 0)
		    lastNonzeroCol = col;

	// Create a trimmed intensity matrix from the intensity
	// matrix by deleting rows and columns that are all 0.

	int numIntensityRows = lastNonzeroRow - firstNonzeroRow + 1;
	int numIntensityCols = lastNonzeroCol - firstNonzeroCol + 1;

	int[][] intensityMatrix
	    = new int[numIntensityRows + 1][numIntensityCols + 1];

	for (int row = firstNonzeroRow; row <= lastNonzeroRow; row++)

	    for (int col = firstNonzeroCol; col <= lastNonzeroCol; col++)

		intensityMatrix[row - firstNonzeroRow + 1]
		    [col - firstNonzeroCol + 1]
		    = untrimmedIntensityMatrix[row][col];

	return intensityMatrix;
    }

    // ------------------------------------------------------------------------

    public static int[][] getDifferenceMatrix(int matrix[][]) {

	int numRows = matrix.length - 1;
	int numCols = matrix[0].length - 1;

	int[][] differenceMatrix = new int[numRows + 1][numCols + 2];

	for (int row = 1; row <= numRows; row++) {

	    differenceMatrix[row][1] = matrix[row][1];

	    for (int col = 1; col <= numCols + 1; col++)

		differenceMatrix[row][col]

		    = ((col == numCols + 1) ? 0 : matrix[row][col])
		    - matrix[row][col - 1];
	}
	return differenceMatrix;
    }

    // ------------------------------------------------------------------------

    public static String toString(int[][] matrix) {

	// Print column labels.

	String s = "\t";

	for (int col = 1; col < matrix[0].length; col++)

	    s += col + "\t";

	s += "\n";

	// Print row labels and rows.

	for (int row = 1; row < matrix.length; row++) {

	    s += row + "\t";

	    for (int col = 1; col < matrix[0].length; col++) {

		// Print . instead of 0 so that nonzero numbers stand
		// out.

		s += (matrix[row][col] == 0) ? "." : ("" + matrix[row][col]);
		s += "\t";
	    }
	    s += "\n";
	}
	s += "\n";
	return s;
    }

    public static void printMatrix(int[][] matrix) {

	System.out.println(toString(matrix));
    }

    // ------------------------------------------------------------------------

    private static void fatalError(Object object) {

	System.out.println(object);
	System.exit(1);
    }

    // ------------------------------------------------------------------------

    public static void main(String[] args) {

	if (args.length != 1)
	    fatalError("usage: java Matrix inputFile");

	int[][] intensityMatrix = getIntensityMatrix(args[0]);
	System.out.println("intensity matrix:");
	printMatrix(intensityMatrix);

	int[][] differenceMatrix = getDifferenceMatrix(intensityMatrix);
	System.out.println("difference matrix:");
	printMatrix(differenceMatrix);
    }
}
