;; Winston C. Yang
;; winston@cs.wisc.edu

;; Changes

;; WCY 2003-01-05
;; WCY 2002-12-03
;; WCY 2002-06-30
;; WCY 2002-06-28
;; WCY 2002-06-23
;; WCY 2002-06-17
;; WCY 2002-06-10
;; WCY 2002-06-03
;; WCY 2002-05-20
;; WCY 2001-11-12
;; WCY 2001-10-15
;; WCY 2001-10-11
;; WCY 2001-10-10
;; WCY 2001-09-27
;; WCY 2001-09-15
;; WCY 2001-09-08
;; WCY 2001-07-07
;; WCY 2001-06-08 (created)

;; Do syntax coloring for AMPL files edited by Emacs, using regular
;; expressions. This is a quick hack; there are a few bugs: an
;; underscore followed by a keyword is not recognized as an
;; identifier, and there might be some trouble in processing large
;; files.

;; --------------------------------------------------------------------------

;; define AMPL mode

(define-derived-mode ampl-mode
  text-mode "Ampl"
  "Major mode for ampl code. Email Winston C. Yang at
winston@cs.wisc.edu \\{ampl-mode-map}")

(add-hook 'ampl-mode-hook 'turn-on-font-lock)

;; --------------------------------------------------------------------------

(defun add (regexp match-index font-face)
  (font-lock-add-keywords
   'ampl-mode
   (list
    (list
     regexp match-index font-face t))))

;; --------------------------------------------------------------------------

;; comments can begin with # and end with newline

(add "\\(#.*\\)" 1 font-lock-comment-face)

;; comments can begin with /* and end with */. There seems to be no
;; regular expression for this comment style that will extend across
;; lines and stops the comment at the first */

(add "\\(/[*].*[*]/\\)" 1 font-lock-comment-face)

;; --------------------------------------------------------------------------

;; strings (single quote and double quote) cannot extend across newlines

(add "\\('[^'\n]*'\\)" 1 font-lock-string-face)

(add "\\(\"[^\"\n]*\"\\)" 1 font-lock-string-face)

;; --------------------------------------------------------------------------

;; declarations (keyword part)

(add "\\<\\(set\\|var\\|param\\|node\\|arc\\)[ \t]+\\([_a-zA-Z0-9]*\\)"

     1 font-lock-keyword-face)

;; declarations (identifier part)

(add "\\<\\(set\\|var\\|param\\|node\\|arc\\)[ \t]+\\([_a-zA-Z0-9]*\\)"

     2 font-lock-variable-name-face)

;; --------------------------------------------------------------------------

;; keywords

(add "\\<\\(param\\|set\\|var\\|from\\|to\\|obj\\|check\\|net_in\\|net_out\\|complements\\|circular\\|ordered\\|logical\\|solver\\|problem\\|for\\|repeat\\|break\\|end\\|minimize\\|maximize\\|continue\\|until\\|while\\|purge\\)[^a-zA-Z0-9_]"

     1 font-lock-keyword-face)

;; set keywords

(add "\\<\\(card\\|next\\|nextw\\|prev\\|prevw\\|first\\|last\\|member\\|ord\\|ord0\\)[^a-zA-Z0-9_]"

    1 font-lock-keyword-face)

;; command keywords

(add "\\<\\(close\\|data\\|display\\|drop\\|end\\|fix\\|include\\|let\\|model\\|objective\\|option\\|print\\|printf\\|quit\\|reset\\|restore\\|shell\\|solution\\|solve\\|update\\|unfix\\|write\\)[^a-zA-Z0-9_]"

    1 font-lock-keyword-face)

;; reserved and predefined words

(add "\\<\\(Infinity\\|binary\\|by\\|data\\|default\\|dimen\\|else\\|exists\\|forall\\|if\\|in\\|integer\\|less\\|max\\|min\\|setof\\|sum\\|symbolic\\|then\\|union\\|within\\)[^a-zA-Z0-9_]"

    1 font-lock-keyword-face)

;; synonyms

(add "\\<\\(and\\|coeff?\\|diff\\|difference\\|dimen\\|dimension\\|floor\\|int\\|inter\\|intersect\\|intersection\\|not\\|or\\|prod\\|product\\|subj\\|subj[ \t]+to\\|subject[ \t]+to\\|s[.]t[.]\\)[^a-zA-Z0-9_]"

    1 font-lock-keyword-face)

;; additional keywords

(add "\\<\\(Current\\|IN\\|INOUT\\|Initial\\|LOCAL\\|OUT\\|all\\|contains\\|environ\\|option\\|suffix\\|table\\)[^a-zA-Z0-9_]"

    1 font-lock-keyword-face)

;; mod

(add "[^.]\\(mod\\)[^a-zA-Z0-9_]" 1 font-lock-keyword-face)

;; (tr) means transpose. Parentheses are required.

(add "(\\([ \t]*tr[ \t]*\\))" 1 font-lock-keyword-face)

;; --------------------------------------------------------------------------

;; built-in functions

(add "\\<\\(abs\\|acos\\|acosh\\|alias\\|asin\\|asinh\\|atan\\|atan2\\|atanh\\|ceil\\|cos\\|exp\\|floor\\|log\\|log10\\|max\\|min\\|precision\\|round\\|sin\\|sinh\\|sqrt\\|tan\\|tanh\\|trunc\\)[^a-zA-Z0-9_]"

    1 font-lock-builtin-face)

;; built-in variables

(add "\\<\\(log_file\\)[^a-zA-Z0-9_]" 1 font-lock-builtin-face)

;; --------------------------------------------------------------------------

;; dot suffixes for variables

(add "[.]\\(astatus\\|defeqn\\|dual\\|init\\|init0\\|lb\\|lb0\\|lb1\\|lb2\\|lrc\\|lslack\\|rc\\|slack\\|sstatus\\|status\\|ub\\|ub0\\|ub1\\|ub2\\|urc\\|uslack\\|val\\)[^a-zA-Z0-9_]"

    1 font-lock-builtin-face)

;; dot suffixes for constraints

(add "[.]\\(astatus\\|body\\|defvar\\|dinit\\|dinit0\\|dual\\|lb\\|lbs\\|ldual\\|lslack\\|slack\\|sstatus\\|status\\|ub\\|ubs\\|udual\\|uslack\\)[^a-zA-Z0-9_]"

    1 font-lock-builtin-face)

;; dot suffixes for objectives

(add "[.]\\(astatus\\|exitcode\\|message\\|result\\|val\\)[^a-zA-Z0-9_]"

    1 font-lock-builtin-face)

;; dot suffixes for problems

(add "[.]\\(astatus\\|exitcode\\|message\\|result\\)[^a-zA-Z0-9_]"

    1 font-lock-builtin-face)

;; --------------------------------------------------------------------------

;; problems

(add "\\<\\(problem[ ]*[_a-zA-Z0-9]*:\\)" 1 font-lock-warning-face)

;; problem variables

(add "\\<\\(problem[ ]*[_a-zA-Z0-9]*:\\)\\([^;]*\\);"

     2 font-lock-variable-name-face)

;; --------------------------------------------------------------------------
;; constraint names

(add "\\<\\(subj[ \t]+to\\|subj\\|subject[ \t]*to\\|s[.]t[.]\\)[ \t]+\\([a-zA-Z0-9_]*\\)"

    2 font-lock-warning-face)

;; names

(add "\\<\\(minimize\\|maximize\\)[ \t]+\\([a-zA-Z0-9_]*\\)"

    2 font-lock-warning-face)

;; --------------------------------------------------------------------------

;; add ampl mode to the list of major modes. An ampl file ends with
;; `.mod' or `.dat' or `.amp' or `.run'

(setq auto-mode-alist
      (append
       '(("\\.mod\\'" . ampl-mode)
	 ("\\.dat\\'" . ampl-mode)
	 ("\\.amp\\'" . ampl-mode)
	 ("\\.run\\'" . ampl-mode))
       auto-mode-alist))
