;; Winston C. Yang
;; winston@cs.wisc.edu
;; Modified 2001-12-07
;; Modified 2001-11-12
;; Modified 2001-10-19
;; Modified 2001-10-10
;; Modified 2001-09-06
;; Modified 2001-07-03
;; Created 2001-06-08

;; Do syntax coloring for GAMS files edited by Emacs, using regular
;; expressions. This is a quick hack; there are a few bugs: an
;; underscore followed by a keyword is not recognized as an
;; identifier, and there might be some trouble in processing large
;; files.

;;---------------------------------------------------------------------------
;; define GAMS mode
(define-derived-mode gams-mode
  text-mode "gams"
  "Major mode for gams code. Email Winston C. Yang at
winston@cs.wisc.edu

\\{gams-mode-map}")

(add-hook 'gams-mode-hook 'turn-on-font-lock)

;;---------------------------------------------------------------------------
;; compile-time commands begin with $ (which must be at the beginning
;; of the line) and end at newline.
(font-lock-add-keywords
 'gams-mode
 '(("\\(^$.*\\)"
    1 font-lock-warning-face t)))

;;---------------------------------------------------------------------------
;; comments can begin with * (which must be at the beginning of the
;; line) and end at newline.
(font-lock-add-keywords
 'gams-mode
 '(("\\(^*.*\\)"
    1 font-lock-comment-face t)))

;; comments can begin with $ontext and end with $offtext
(font-lock-add-keywords
 'gams-mode
 '(("^$[ \t]*[oO][nN][tT][eE][xX][tT][ \t]*.*\n\\(.\\|\n\\)*\n$[ \t]*[oO][fF][fF][tT][eE][xX][tT][ \t]*.*"
    0 font-lock-comment-face t)))

;; comments can begin with // and end with newline (this does not
;; handle the case where the // is after a ``put'' statement; the //
;; are interpreted as 2 newlines)
(font-lock-add-keywords
 'gams-mode
 '(("\\(//.*\\)"
    1 font-lock-comment-face t)))

;;---------------------------------------------------------------------------
;; strings cannot extend across newlines
(font-lock-add-keywords
 'gams-mode
 '(("\\('[^'\n]*'\\)"
    1 font-lock-string-face t)))

(font-lock-add-keywords
 'gams-mode
 '(("\\(\"[^\"\n]*\"\\)"
    1 font-lock-string-face t)))
;;---------------------------------------------------------------------------
;; constraints begin with a letter or underscore and end at ..
(font-lock-add-keywords
 'gams-mode
 '(("\\([_a-zA-Z].*[.][.]\\)"
    1 font-lock-warning-face t)))

;;---------------------------------------------------------------------------
;; keywords

;; Page 62 in GAMS manual:
;; free,positive,negative,binary,integer
(font-lock-add-keywords
 'gams-mode
 '(("\\(\\<\\)\\([fF][rR][eE][eE]\\|[pP][oO][sS][iI][tT][iI][vV][eE]\\|[nN][eE][gG][aA][tT][iI][vV][eE]\\|[bB][iI][nN][aA][rR][yY]\\|[iI][nN][tT][eE][gG][eE][rR]\\)[^a-zA-Z0-9_]"
    2 font-lock-keyword-face t)))

;; Page 33 in GAMS manual:
;; abort,acronyms?,alias,all,and,assign,binary,card,diag,display,eps,eq,equations?,ge,gt,inf,integer,le,loop,lt,maximizing,minimizing,models?,na,ne,negative,not,options?,or,ord,parameters?,positive,prod,sameas,scalars?,sets?,smax,smin,sos1,sos2,sum,system,table,using,variables?,xor,yes,repeat,until,while,if,then,else,semicont,semiint,files?,putpage,free,no,solve,for
(font-lock-add-keywords
 'gams-mode
 '(("\\(\\<\\)\\([aA][bB][oO][rR][tT]\\|[aA][cC][rR][oO][nN][yY][mM][sS]?\\|[aA][lL][iI][aA][sS]\\|[aA][lL][lL]\\|[aA][nN][dD]\\|[aA][sS][sS][iI][gG][nN]\\|[bB][iI][nN][aA][rR][yY]\\|[cC][aA][rR][dD]\\|[dD][iI][aA][gG]\\|[dD][iI][sS][pP][lL][aA][yY]\\|[eE][pP][sS]\\|[eE][qQ]\\|[eE][qQ][uU][aA][tT][iI][oO][nN][sS]?\\|[gG][eE]\\|[gG][tT]\\|[iI][nN][fF]\\|[iI][nN][tT][eE][gG][eE][rR]\\|[lL][eE]\\|[lL][oO][oO][pP]\\|[lL][tT]\\|[mM][aA][xX][iI][mM][iI][zZ][iI][nN][gG]\\|[mM][iI][nN][iI][mM][iI][zZ][iI][nN][gG]\\|[mM][oO][dD][eE][lL][sS]?\\|[nN][aA]\\|[nN][eE]\\|[nN][eE][gG][aA][tT][iI][vV][eE]\\|[nN][oO][tT]\\|[oO][pP][tT][iI][oO][nN][sS]?\\|[oO][rR]\\|[oO][rR][dD]\\|[pP][aA][rR][aA][mM][eE][tT][eE][rR][sS]?\\|[pP][oO][sS][iI][tT][iI][vV][eE]\\|[pP][rR][oO][dD]\\|[sS][aA][mM][eE][aA][sS]\\|[sS][cC][aA][lL][aA][rR][sS]?\\|[sS][eE][tT][sS]?\\|[sS][mM][aA][xX]\\|[sS][mM][iI][nN]\\|[sS][oO][sS]1\\|[sS][oO][sS]2\\|[sS][uU][mM]\\|[sS][yY][sS][tT][eE][mM]\\|[tT][aA][bB][lL][eE]\\|[uU][sS][iI][nN][gG]\\|[vV][aA][rR][iI][aA][bB][lL][eE][sS]?\\|[xX][oO][rR]\\|[yY][eE][sS]\\|[rR][eE][pP][eE][aA][tT]\\|[uU][nN][tT][iI][lL]\\|[wW][hH][iI][lL][eE]\\|[iI][fF]\\|[tT][hH][eE][nN]\\|[eE][lL][sS][eE]\\|[sS][eE][mM][iI][cC][oO][nN][tT]\\|[sS][eE][mM][iI][iI][nN][tT]\\|[fF][iI][lL][eE][sS]?\\|[pP][uU][tT][pP][aA][gG][eE]\\|[fF][rR][eE][eE]\\|[nN][oO]\\|[sS][oO][lL][vV][eE]\\|[fF][oO][rR]\\|[mM][aA][xX]\\|[mM][iI][nN]\\|[pP][uU][tT]\\|[pP][uU][tT][cC][lL][oO][sS][eE]\\)[^a-zA-Z0-9_]"
    2 font-lock-keyword-face t)))

;;---------------------------------------------------------------------------
;; built-in functions

;; Page 71 in GAMS manual:
(font-lock-add-keywords
 'gams-mode
 '(("\\(\\<\\)\\(abs\\|arctan\\|ceil\\|cos\\|errorf\\|exp\\|floor\\|log\\|log10\\|mapval\\|max\\|min\\|mod\\|normal\\|power\\|round\\|sign\\|sin\\|sqr\\|sqrt\\|trunc\\|uniform\\)[^a-zA-Z0-9_]"
    2 font-lock-builtin-face t)))

;; Page 68 in GAMS manual:
;; =e=, =e=, =g=, =n=
(font-lock-add-keywords
 'gams-mode
 '(("\\(=[elgnELGN]=\\)"
    1 font-lock-builtin-face t)))

;; suffices (come after a period)
(font-lock-add-keywords
 'gams-mode
 '(("[_a-zA-Z]+[_a-zA-Z0-9]*[.]\\([_a-zA-Z]+[_a-zA-Z0-9]*\\)"
    1 font-lock-builtin-face t)))

;; limrow, limcol, solprint, on, off
(font-lock-add-keywords
 'gams-mode
 '(("\\(\\<\\)\\([lL][iI][mM][rR][oO][wW]\\|[lL][iI][mM][cC][oO][lL]\\|[sS][oO][lL][pP][rR][iI][nN][tT]\\|[oO][nN]\\|[oO][fF][fF]\\)[^a-zA-Z0-9_]"
    2 font-lock-builtin-face t)))

;; model type: cns, dnlp, lp, mcp, minlp, mip, mpec, nlp, rminlp, rmip
(font-lock-add-keywords
 'gams-mode
 '(("\\(\\<\\)\\([cC][nN][sS]\\|[dD][nN][lL][pP]\\|[lL][pP]\\|[mM][cC][pP]\\|[mM][iI][nN][lL][pP]\\|[mM][iI][pP]\\|[mM][pP][eE][cC]\\|[nN][lL][pP]\\|[rR][mM][iI][nN][lL][pP]\\|[rR][mM][iI][pP]\\)[^a-zA-Z0-9_]"
    2 font-lock-builtin-face t)))

;;---------------------------------------------------------------------------
;; declarations (identifier part)
(font-lock-add-keywords
 'gams-mode
 '(("\\<\\(\\<[sS][eE][tT][sS]?\\|[vV][aA][rR][iI][aA][bB][lL][eE][sS]?\\|[pP][aA][rR][aA][mM][eE][tT][eE][rR][sS]?\\|[sS][cC][aA][lL][aA][rR][sS]?\\|[eE][qQ][uU][aA][tT][iI][oO][nN][sS]?\\)[ \t]+\\([_a-zA-Z0-9]+\\)"
    2 font-lock-variable-name-face t)))

;;---------------------------------------------------------------------------
;; indices (within parentheses)
(font-lock-add-keywords
 'gams-mode
 '(("(\\([-+_a-zA-Z0-9, \t\"']+\\))"
    1 font-lock-type-face t)))

; ;; indices, corresponding identifier, and parentheses
; (font-lock-add-keywords
;  'gams-mode
;  '(("\\([_a-zA-Z0-9]+[ \t]*([-+_a-zA-Z0-9, \t\"']+)\\)"
;     1 font-lock-type-face t)))

;;---------------------------------------------------------------------------
;; add gams-mode to the list of major modes. A gams file ends with
;; `.gms'
(setq auto-mode-alist
      (append
       '(("\\.gms\\'" . gams-mode))
       auto-mode-alist))
